# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'Article'
        db.create_table('demo_cms_article', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(db_index=True, unique=True, max_length=100, blank=True)),
            ('title', self.gf('html_field.db.models.fields.HTMLField')(default=u'Page title')),
            ('content', self.gf('django.db.models.fields.TextField')(default=u'Page content')),
            ('publication', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('template', self.gf('django.db.models.fields.CharField')(default='', max_length=200, blank=True)),
            ('logo', self.gf('django.db.models.fields.files.ImageField')(default='', max_length=100, null=True, blank=True)),
            ('temp_logo', self.gf('django.db.models.fields.files.ImageField')(default='', max_length=100, null=True, blank=True)),
            ('summary', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(default=None, related_name='demo_cms_article_rel', null=True, blank=True, to=orm['coop_cms.ArticleCategory'])),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(default=None, to=orm['auth.User'], null=True, blank=True)),
        ))
        db.send_create_signal('demo_cms', ['Article'])


    def backwards(self, orm):

        # Deleting model 'Article'
        db.delete_table('demo_cms_article')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'coop_cms.articlecategory': {
            'Meta': {'object_name': 'ArticleCategory'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'demo_cms.article': {
            'Meta': {'object_name': 'Article'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {'default': "u'Page content'"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logo': ('django.db.models.fields.files.ImageField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'publication': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'related_name': "'demo_cms_article_rel'", 'null': 'True', 'blank': 'True', 'to': "orm['coop_cms.ArticleCategory']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '100', 'blank': 'True'}),
            'summary': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'temp_logo': ('django.db.models.fields.files.ImageField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'title': ('html_field.db.models.fields.HTMLField', [], {'default': "u'Page title'"})
        }
    }

    complete_apps = ['demo_cms']



# -*- coding: utf-8 -*-

from unittest import skipIf

from django.conf import settings
from django.contrib.sites.models import Site
from django.contrib.sitemaps.views import sitemap as sitemap_view
from django.core.urlresolvers import reverse
from django.test.client import RequestFactory
from django.utils.text import slugify
from django.utils.translation import activate

from model_mommy import mommy

from coop_cms.models import BaseArticle, SiteSettings
from coop_cms.settings import get_article_class, is_localized, is_multilang
from coop_cms.sitemap import get_sitemaps
from coop_cms.tests import BaseTestCase
from coop_cms.utils import get_url_in_language


class SitemapTest(BaseTestCase):
    
    def setUp(self):
        self._site2 = mommy.make(Site, id=settings.SITE_ID+1)

    def test_sitemap_empty(self):
        url = reverse("coop_cms_sitemap")
        factory = RequestFactory()
        request = factory.get(url)
        response = sitemap_view(request, get_sitemaps())
        self.assertEqual(200, response.status_code)
    
    def test_sitemap(self):
        site = Site.objects.get_current()
        site2 = self._site2
        
        article_class = get_article_class()
        
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT)
        
        article2.sites.add(site2)
        article2.save()
        
        article3.sites.remove(site)
        article3.sites.add(site2)
        article3.save()

        factory = RequestFactory()
        request = factory.get('/sitemap.xml')
        response = sitemap_view(request, get_sitemaps())

        self.assertEqual(200, response.status_code)

        self.assertContains(response, site.domain+article1.get_absolute_url())
        self.assertContains(response, site.domain+article2.get_absolute_url())
        self.assertNotContains(response, article3.get_absolute_url())
        self.assertNotContains(response, article4.get_absolute_url())

    def test_sitemap_only_site(self):
        site = Site.objects.get_current()
        site2 = self._site2

        mommy.make(SiteSettings, site=site, sitemap_mode=SiteSettings.SITEMAP_ONLY_SITE)

        article_class = get_article_class()

        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT)

        article2.sites.add(site2)
        article2.save()

        article3.sites.remove(site)
        article3.sites.add(site2)
        article3.save()

        factory = RequestFactory()
        request = factory.get('/sitemap.xml')
        response = sitemap_view(request, get_sitemaps())

        self.assertEqual(200, response.status_code)

        self.assertContains(response, site.domain+article1.get_absolute_url())
        self.assertContains(response, site.domain+article2.get_absolute_url())
        self.assertNotContains(response, article3.get_absolute_url())
        self.assertNotContains(response, article4.get_absolute_url())

    def test_sitemap_all(self):
        site = Site.objects.get_current()
        site2 = self._site2

        site_settings = mommy.make(SiteSettings, site=site, sitemap_mode=SiteSettings.SITEMAP_ALL)

        article_class = get_article_class()

        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT)

        article2.sites.add(site2)
        article2.save()

        article3.sites.remove(site)
        article3.sites.add(site2)
        article3.save()

        factory = RequestFactory()
        request = factory.get('/sitemap.xml')
        response = sitemap_view(request, get_sitemaps())

        self.assertEqual(200, response.status_code)

        self.assertContains(response, site.domain+article1.get_absolute_url())
        self.assertContains(response, site.domain+article2.get_absolute_url())
        self.assertContains(response, site2.domain+article3.get_absolute_url())
        self.assertNotContains(response, article4.get_absolute_url())


class LocaleSitemapTest(SitemapTest):
    """test sitemap can be localized"""

    def setUp(self):
        super(LocaleSitemapTest, self).setUp()
        activate(settings.LANGUAGES[0][0])

    def tearDown(self):
        activate(settings.LANGUAGES[0][0])

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_sitemap_lang(self):
        site = Site.objects.get_current()
        site2 = self._site2

        article_class = get_article_class()

        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT)

        article2.sites.add(site2)
        article2.save()

        article3.sites.remove(site)
        article3.sites.add(site2)
        article3.save()

        factory = RequestFactory()
        request = factory.get('/sitemap.xml')
        response = sitemap_view(request, get_sitemaps())
        self.assertEqual(200, response.status_code)

        self.assertContains(response, site.domain + article1.get_absolute_url())
        self.assertContains(response, site.domain + article2.get_absolute_url())
        self.assertNotContains(response, article3.get_absolute_url())
        self.assertNotContains(response, article4.get_absolute_url())

        for (lang, name) in settings.LANGUAGES:
            self.assertContains(response, site.domain + get_url_in_language(article1.get_absolute_url(), lang))
            self.assertContains(response, site.domain + get_url_in_language(article2.get_absolute_url(), lang))
            self.assertNotContains(response, get_url_in_language(article3.get_absolute_url(), lang))
            self.assertNotContains(response, get_url_in_language(article4.get_absolute_url(), lang))

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_sitemap_lang_slug(self):
        """Test that the url is in the locale language"""
        site = Site.objects.get_current()
        from modeltranslation.utils import build_localized_fieldname  # pylint: disable=F0401

        kwargs1 = {}
        kwargs2 = {}

        for lang_code, lang_name in settings.LANGUAGES:
            loc_title_var = build_localized_fieldname('title', lang_code)
            loc_slug_var = build_localized_fieldname('slug', lang_code)
            kwargs1[loc_title_var] = u'article-{0}-1'.format(lang_name)
            kwargs2[loc_title_var] = u'other-article-{0}-2'.format(lang_name)
            kwargs1[loc_slug_var] = slugify(kwargs1[loc_title_var])
            kwargs2[loc_slug_var] = slugify(kwargs2[loc_title_var])

        article_class = get_article_class()

        article1 = mommy.make(article_class, publication=BaseArticle.PUBLISHED, **kwargs1)
        article2 = mommy.make(article_class, publication=BaseArticle.PUBLISHED, **kwargs2)

        factory = RequestFactory()
        request = factory.get('/sitemap.xml')
        response = sitemap_view(request, get_sitemaps())
        self.assertEqual(200, response.status_code)

        for (lang, name) in settings.LANGUAGES:
            activate(lang)
            self.assertContains(response, site.domain + article1.get_absolute_url())
            self.assertContains(response, site.domain + article2.get_absolute_url())



# -*- coding: utf-8 -*-
"""generic views"""

from django.contrib.contenttypes.models import ContentType
from django.contrib.messages.api import success as success_message, error as error_message
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.forms.models import modelformset_factory
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.utils.translation import ugettext as _
from django.views.generic import TemplateView
from django.views.generic.base import View
from django.views.generic.list import ListView as DjangoListView

from coop_html_editor import utils as html_editor_utils

from coop_cms.logger import logger


class ListView(DjangoListView):
    """generic list view"""
    ordering = ''
    
    def get_context_data(self, **kwargs):
        context = super(ListView, self).get_context_data(**kwargs)
        context['model'] = self.model
        return context
    
    def get_queryset(self):
        if self.ordering:
            if type(self.ordering) in (list, tuple):
                return self.model.objects.order_by(*self.ordering)
            else:
                return self.model.objects.order_by(self.ordering)
        else:
            return self.model.objects.all()


class EditableObjectView(View):
    """Base class for html-inline editable objects"""
    model = None
    template_name = ""
    form_class = None
    field_lookup = "pk"
    edit_mode = False
    varname = "object"
    object = None
    form = None

    def can_edit_object(self):
        """check edit perms"""
        can_edit_perm = 'can_edit_{0}'.format(self.varname)
        user = self.request.user
        return user.is_authenticated() and user.is_active and user.has_perm(can_edit_perm, self.object)
        
    def can_access_object(self):
        """check access perms: 404 if not"""
        return True
    
    def can_view_object(self):
        """check view perm: 403 if not"""
        if self.edit_mode:
            return self.can_edit_object()
        else:
            can_view_perm = 'can_view_{0}'.format(self.varname)
            return self.request.user.has_perm(can_view_perm, self.object)
    
    def get_object(self):
        """get the object to edit"""
        lookup = {self.field_lookup: self.kwargs[self.field_lookup]}
        return get_object_or_404(self.model, **lookup)
    
    def get_context_data(self):
        """template context"""
        return {
            'form': self.form if self.edit_mode else None,
            'editable': self.can_edit_object(),
            'edit_mode': self.edit_mode,
            'title': getattr(self.object, 'title', unicode(self.object)),
            'coop_cms_edit_url': self.get_edit_url(),
            'coop_cms_cancel_url': self.get_cancel_url(),
            'coop_cms_can_view_callback': self.can_view_object,
            'coop_cms_can_edit_callback': self.can_edit_object,
            self.varname: self.object,
            'raw_'+self.varname: self.object,
        }
        
    def get_edit_url(self):
        """url for object editing"""
        return self.object.get_edit_url()
    
    def get_cancel_url(self):
        """url after cancel editing"""
        if hasattr(self.object, 'get_cancel_url'):
            return self.object.get_cancel_url()
        else:
            return self.object.get_absolute_url()
        
    def get_template(self):
        """get template"""
        return self.template_name
    
    def handle_object_not_found(self):
        """called if object not found"""
        pass

    def get_form_class(self):
        return self.form_class

    def get_form_kwargs(self):
        return {}

    def get_form(self, *args, **kwargs):
        kwargs.update(self.get_form_kwargs())
        form_class = self.get_form_class()
        return form_class(*args, **kwargs)

    def get(self, request, *args, **kwargs):
        """handle http get -> view"""
        try:
            self.object = self.get_object()
        except Http404:
            return_this = self.handle_object_not_found()
            if return_this:
                return return_this
            else:
                raise
        
        if not self.can_access_object():
            raise Http404
        
        if not self.can_view_object():
            logger.warning("PermissionDenied")
            raise PermissionDenied
        
        self.form = self.get_form(instance=self.object)
        
        return render(
            request,
            self.get_template(),
            self.get_context_data()
        )
    
    def after_save(self, object):
        """called after save"""
        pass
    
    def post(self, request, *args, **kwargs):
        """handle http post -> edit"""
        if not self.edit_mode:
            raise Http404
        
        self.object = self.get_object()
        
        if not self.can_edit_object():
            logger.warning("PermissionDenied")
            raise PermissionDenied

        self.form = self.get_form(request.POST, request.FILES, instance=self.object)

        forms_args = html_editor_utils.extract_forms_args(request.POST)
        inline_html_forms = html_editor_utils.make_forms(forms_args, request.POST)

        if self.form.is_valid() and all([_form.is_valid() for _form in inline_html_forms]):

            self.object = self.form.save()
            
            self.after_save(self.object)
            
            if inline_html_forms:
                [_the_form.save() for _the_form in inline_html_forms]

            success_message(request, _(u'The object has been saved properly'))

            return HttpResponseRedirect(self.object.get_absolute_url())
        else:
            error_text = u'<br />'.join([unicode(_form.errors) for _form in [self.form]+inline_html_forms if _form.errors])
            error_message(request, _(u'An error occured: {0}').format(error_text))
            logger.debug(u"error: {0}".format(error_text))
    
        return render(
            request,
            self.get_template(),
            self.get_context_data()
        )


class EditableFormsetView(TemplateView):
    """Base class for editing several objects on the same page"""
    template_name = ""
    model = None
    form_class = None
    extra = 1
    edit_mode = False
    success_url = ""
    success_view_name = ""
    formset = None
    
    def can_edit_objects(self):
        """check edit perms"""
        ct = ContentType.objects.get_for_model(self.model)
        can_edit_perm = '{0}.change_{1}'.format(ct.app_label, ct.model)
        user = self.request.user
        return user.is_authenticated() and user.is_active and user.has_perm(can_edit_perm, None)
        
    def can_view_objects(self):
        """check view perms"""
        if self.edit_mode:
            return self.can_edit_objects()
        return True
    
    def get_context_data(self):
        """template context"""
        context = {
            'editable': True,
            'edit_mode': self.edit_mode,
            'coop_cms_edit_url': self.get_edit_url(),
            'coop_cms_cancel_url': self.get_cancel_url(),
            'coop_cms_can_view_callback': self.can_view_objects,
            'coop_cms_can_edit_callback': self.can_edit_objects,
            'objects': self.get_queryset(),
            'raw_objects': self.get_queryset(),
        }
        if self.edit_mode:
            context['formset'] = self.formset
        return context
        
    def get_form_class(self):
        """get form class"""
        return self.form_class
    
    def get_queryset(self):
        """get objects"""
        return self.model.objects.all()
    
    def get_template(self):
        """get template"""
        return self.template_name
    
    def get_cancel_url(self):
        """url to go after cancel edition"""
        return self.get_success_url()
    
    def get_edit_url(self):
        """url for editing object"""
        return ''
        
    def get_success_url(self):
        """url to go after edition success"""
        return self.success_url or reverse(self.success_view_name) if self.success_view_name else ''
    
    def get_formset_class(self):
        """formset class"""
        return modelformset_factory(self.model, self.get_form_class(), extra=self.extra)
    
    def get_formset(self, *args, **kwargs):
        """formset"""
        Formset = self.get_formset_class()
        return Formset(queryset=self.get_queryset(), *args, **kwargs)
    
    def get(self, request, *args, **kwargs):
        """handle http get --> view"""
        if not self.can_view_objects():
            raise PermissionDenied
            
        self.formset = self.get_formset()
        return render(
            request,
            self.get_template(),
            self.get_context_data()
        )
    
    def _pre_save_object(self, form):
        """before saving an object"""
        return True
    
    def _post_save_object(self, obj, form):
        """after saving an object"""
        pass
    
    def post(self, request, *args, **kwargs):
        """handle http post -> edit"""
        if not self.edit_mode:
            raise Http404
    
        if not self.can_edit_objects():
            raise PermissionDenied
        
        self.formset = self.get_formset(request.POST, request.FILES)
        
        forms_args = html_editor_utils.extract_forms_args(request.POST)
        inline_html_forms = html_editor_utils.make_forms(forms_args, request.POST)

        # Handle case where formset post data has value which are not in the queryset
        formset_index_error = False
        try:
            formset_is_valid = self.formset.is_valid()
        except IndexError:
            formset_index_error = True
            formset_is_valid = False

        if formset_is_valid and all([_form.is_valid() for _form in inline_html_forms]):
            for form in self.formset:
                if self._pre_save_object(form):
                    obj = form.save()
                    self._post_save_object(obj, form)
            
            if inline_html_forms:
                [_html_form.save() for _html_form in inline_html_forms]
            
            success_message(request, _(u'The objects have been saved properly'))

            url = self.get_success_url()
            return HttpResponseRedirect(url)
        else:
            if formset_index_error:
                logger.warning(_(u'Index error in formset: some objects may be missing'))
                error_message(request, _(u'An error occured: At least one object is missing. Please try again.'))
                return HttpResponseRedirect(self.get_cancel_url())
            else:
                for _form in self.formset:
                    errors = _form.errors
                    if errors:
                        logger.warning(errors)
        
        return render(
            request,
            self.get_template(),
            self.get_context_data()
        )



# -*- coding: utf-8 -*-
"""
Basic app for article
"""

from django import VERSION

if VERSION > (1, 7, 0):
    from django.apps import AppConfig

    class BasicCmsAppConfig(AppConfig):
        name = 'coop_cms.apps.basic_cms'
        verbose_name = "coop CMS > Basic CMS"



# -*- coding: utf-8 -*-
"""
coop_edition template tags
used for magic form
"""

from django import VERSION as DJANGO_VERSION
from django import template
from django.template import Context
from django.template.base import TextNode, VariableNode
from django.template.context_processors import csrf
from django.template.loader import get_template, TemplateDoesNotExist
from django.template.loader_tags import IncludeNode
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from coop_html_editor.templatetags.html_editor_utils import InlineHtmlEditNode, InlineHtmlMultipleEditNode

from coop_cms.models import PieceOfHtml, BaseArticle, Fragment, FragmentType, FragmentFilter
from coop_cms.settings import get_article_class
from coop_cms.utils import get_text_from_template

register = template.Library()


class DummyEngine(object):
    """Used for monkey patching Context"""
    debug = False
    string_if_invalid = ''


class PieceOfHtmlEditNode(InlineHtmlEditNode):
    """Template node for editing a PieceOfHtml"""

    def render(self, context):
        """convert to html"""
        if context.get('form', None) or context.get('formset', None):
            context.dicts[0]['inline_html_edit'] = True
        return super(PieceOfHtmlEditNode, self).render(context)


@register.tag
def coop_piece_of_html(parser, token):
    """template tag"""
    args = token.split_contents()
    div_id = args[1]
    read_only = False
    extra_id = ""
    if len(args) > 2:
        for item in args[2:]:
            if 0 == item.find("extra_id="):
                extra_id = item.replace("extra_id=", '')
        
        read_only = (args[2] == "read-only")
    
    lookup_args = {'div_id': div_id}
    if extra_id:
        lookup_args.update({'extra_id': extra_id})
    
    return PieceOfHtmlEditNode(PieceOfHtml, lookup_args, 'content', read_only)


class FragmentEditNode(InlineHtmlMultipleEditNode):
    """Template Node for Fragment edition"""

    def __init__(self, lookup, kwargs=None):
        super(FragmentEditNode, self).__init__(Fragment, lookup, 'content')
        self._edit_mode = False
        self.fragment_filter = None
        self.kwargs = kwargs or {}
        self.fragment_type = None

    def _get_objects(self, lookup):
        """get the fragment"""
        self.fragment_type = FragmentType.objects.get_or_create(name=lookup['name'])[0]
        queryset = Fragment.objects.filter(type=self.fragment_type)
        if 'extra_id' in lookup:
            self.fragment_filter = FragmentFilter.objects.get_or_create(extra_id=lookup['extra_id'])[0]
            queryset = queryset.filter(filter=self.fragment_filter)
        return queryset
    
    def _get_object_lookup(self, obj):
        """get object lookup"""
        return {"id": obj.id}

    def _pre_object_render(self, obj):
        """call before rendering an object"""
        return u'<div class="coop-fragment {0}" rel="{1}">'.format(obj.css_class, obj.id)
    
    def _post_object_render(self, obj):
        """call after rendering an object"""
        return u'</div>'
    
    def _object_render(self, idx, obj, context):
        """convert object to html"""
        value = getattr(obj, self._field_name)
        template_name = self.kwargs.get('template_name', '')
        if template_name:
            template_name = self._resolve_arg(template_name, context)
            template_ = get_template(template_name)
            objects_count = self.get_objects_to_render_count()
            object_content = template_.render(template.Context({
                'css_class': obj.css_class,
                'name': obj.name,
                'slug': slugify(obj.name),
                'id': obj.id,
                'index': idx,
                'objects_count': objects_count,
                'fragment': self._render_value(context, self._get_object_lookup(obj), value),
                'form': self._edit_mode
            }))
        else:
            object_content = self._pre_object_render(obj)
            object_content += self._render_value(context, self._get_object_lookup(obj), value)
            object_content += self._post_object_render(obj)
        return object_content
    
    def render(self, context):
        """convert to html"""
        self._edit_mode = False
        if context.get('form', None) or context.get('formset', None):
            context.dicts[0]['inline_html_edit'] = True
            self._edit_mode = True
        html = super(FragmentEditNode, self).render(context)
        filter_id = self.fragment_filter.id if self.fragment_filter else ""
        if self._edit_mode:
            html_layout = u'<div style="display: none; visibility: hidden;" class="coop-fragment-type" '
            html_layout += u'rel="{0}" data-filter="{2}">{1}</div>'
            pre_html = html_layout.format(
                self.fragment_type.id, self.fragment_type.name, filter_id
            )
        else:
            pre_html = u''
        return pre_html + html


@register.tag
def coop_fragments(parser, token):
    """templatetag"""
    args = token.split_contents()
    lookup = {'name': args[1]}
    extra_id_found = False
    if len(args) > 2:
        args2 = args[2]
        if args2.find("=") < 0:
            lookup["extra_id"] = args2
            extra_id_found = True

    kwargs = {}
    start_index = 2 if extra_id_found else 1
    for arg in args[start_index:]:
        if arg.find("=") > 0:
            key, value = arg.split('=')
            if key == "extra_id" and not extra_id_found:
                lookup["extra_id"] = value
                extra_id_found = True
            else:
                kwargs[key] = value
    return FragmentEditNode(lookup, kwargs)


class ArticleSummaryEditNode(InlineHtmlEditNode):
    """edit the article summary"""

    def render(self, context):
        """to html"""
        if context.get('form', None):
            context.dicts[0]['inline_html_edit'] = True
        return super(ArticleSummaryEditNode, self).render(context)


@register.tag
def article_summary_edit(parser, token):
    """template tag"""
    article_class = get_article_class()
    article_id = token.split_contents()[1]
    return ArticleSummaryEditNode(article_class, {'id': article_id}, 'summary')


class ArticleTitleNode(template.Node):
    """article title tag"""

    def render(self, context):
        """to html"""
        is_edition_mode = context.get('form', None) is not None
        article = context.get('article')
        return u"{0}{1}{2}{3}".format(
            article.title,
            _(u" [EDITION]") if is_edition_mode else u"",
            _(u" [DRAFT]") if article.publication == BaseArticle.PUBLISHED else u"",
            _(u" [ARCHIVED]") if article.publication == BaseArticle.ARCHIVED else u"",
        )


@register.tag
def article_title(parser, token):
    """article title tag"""
    return ArticleTitleNode()


class CmsFormMediaNode(template.Node):
    """generate html for getting required js and css"""

    def render(self, context):
        form = context.get('form', None)
        formset = context.get('formset', None)
        if form or formset:
            template_ = template.Template("{{form.media}}")
            html = template_.render(template.Context({'form': form or formset}))
            #django 1.5 fix : " are escaped as &quot; and cause script tag 
            #for aloha to be broken
            return html.replace("&quot;", '"') 
        else:
            return ""


@register.tag
def cms_form_media(parser, token):
    """generate html for getting required js and css"""
    return CmsFormMediaNode()


def _extract_if_node_args(parser, token):
    """utility for if else endif type of tags"""
    nodelist_true = parser.parse(('else', 'endif'))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse(('endif',))
        parser.delete_first_token()
    else:
        nodelist_false = template.NodeList()
    return nodelist_true, nodelist_false


class IfCmsEditionNode(template.Node):
    """Do something if edition mode"""

    def __init__(self, nodelist_true, nodelist_false):
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false

    def __iter__(self):
        for node in self.nodelist_true:
            yield node
        for node in self.nodelist_false:
            yield node

    def _check_condition(self, context):
        """check condition of the if"""
        return context.get('form', None) or context.get('formset', None) 

    def render(self, context):
        """to html"""
        if self._check_condition(context):
            return self.nodelist_true.render(context)
        else:
            return self.nodelist_false.render(context)


@register.tag
def if_cms_edition(parser, token):
    """Do something if edition mode"""
    nodelist_true, nodelist_false = _extract_if_node_args(parser, token)
    return IfCmsEditionNode(nodelist_true, nodelist_false)


class IfNotCmsEditionNode(IfCmsEditionNode):
    """Do something if not edition mode"""
    def _check_condition(self, context):
        return not super(IfNotCmsEditionNode, self)._check_condition(context)


@register.tag
def if_not_cms_edition(parser, token):
    """Do something if not edition mode"""
    nodelist_true, nodelist_false = _extract_if_node_args(parser, token)
    return IfNotCmsEditionNode(nodelist_true, nodelist_false)


CMS_FORM_TEMPLATE = """
<form id="cms_form" enctype="multipart/form-data"  method="POST" action="{{post_url}}">{% csrf_token %}
    {% include "coop_cms/_form_error.html" with errs=form.non_field_errors %}
    {{inner}} <input type="submit" style="display: none"> </form>
"""


class SafeWrapper(object):
    """This manages display of object in edit or non-edit context"""

    def __init__(self, wrapped, logo_size=None, logo_crop=None):
        self._wrapped = wrapped
        self._logo_size = logo_size
        self._logo_crop = logo_crop

    def __getattr__(self, field):
        value = getattr(self._wrapped, field)
        if field == 'logo':
            src = getattr(self._wrapped, 'logo_thumbnail')(False, self._logo_size, self._logo_crop)
            if src:
                try:
                    template_ = get_template("coop_cms/widgets/_img_logo.html")
                    value = template_.render(
                        template.Context(
                            {
                                'url': src.url,
                                'extra_classes': get_text_from_template("coop_cms/widgets/_imageedit_cssclass.html")
                            }
                        )
                    )
                except TemplateDoesNotExist:
                    value = u'<img class="logo" src="{0}" />'.format(src.url)
            else:
                value = u''
            return mark_safe(value)
        elif callable(value):
            try:
                return value()
            except KeyError:
                pass
        elif type(value) in (unicode, str):
            return mark_safe(value)
        return value


class FormWrapper(object):
    """This manages display of object in edit or non-edit context"""

    def __init__(self, form, obj, logo_size=None, logo_crop=None):
        self._form = form
        self._obj = obj
        if logo_size:
            self._form.set_logo_size(logo_size, logo_crop)

    def __getitem__(self, field, logo_size=None):
        """get attribute"""
        if field in self._form.fields.keys():
            template_ = template.Template("""
                    {%% with form.%s.errors as errs %%}
                    {%% include "coop_cms/_form_error.html" %%}{%% endwith %%}{{form.%s}}
                """ % (field, field))
            return template_.render(template.Context({'form': self._form}))
        else:
            return getattr(self._obj, field)


class CmsEditNode(template.Node):
    """cms_edit -> manages edition of object"""

    def __init__(self, nodelist_content, var_name, logo_size=None, logo_crop=None):
        self.var_name = var_name
        self.nodelist_content = nodelist_content
        self._logo_size = logo_size.strip("'").strip('"') if logo_size else None
        self._logo_crop = logo_crop.strip("'").strip('"') if logo_crop else None
        self._render_logo_size = self._logo_size and (self._logo_size == logo_size)
        self._render_logo_crop = self._logo_crop and (self._logo_crop == logo_crop)
        self.post_url = ""

    def __iter__(self):
        for node in self.nodelist_content:
            yield node

    def _render_nodes(self, context, inner_context, safe_context):
        """Replace nested nodes with proper content"""
        managed_node_types = [
            TextNode,
            template.defaulttags.IfNode,
            IfCmsEditionNode,
            IfNotCmsEditionNode,
            template.defaulttags.ForNode,
        ]

        nodes_content = ""
        for node in self.nodelist_content:

            if any([isinstance(node, node_type) for node_type in managed_node_types]):
                local_context = Context(safe_context)
                if hasattr(context, 'template'):
                    local_context.template = context.template
                content = node.render(local_context)

            elif node.__class__.__name__ == 'MediaListNode':
                content = node.render(context)
                safe_context[node.var_name] = context.get(node.var_name)
                inner_context[node.var_name] = context.get(node.var_name)

            elif node.__class__.__name__ == 'AssignmentNode':
                content = node.render(context)
                safe_context[node.target_var] = context.get(node.target_var)
                inner_context[node.target_var] = context.get(node.target_var)

            elif DJANGO_VERSION >= (1, 8, 0) and isinstance(node, IncludeNode):
                # monkey patching for django 1.8
                template_name = node.template.resolve(context)
                node.template = get_template(template_name)
                node.template.resolve = lambda s, c: s
                context_dict = inner_context.copy()
                if node.extra_context:
                    for filter_expression in node.extra_context:
                        value = node.extra_context[filter_expression].resolve(context)
                        context_dict[filter_expression] = value
                the_context = Context(context_dict)
                the_context.template = node.template
                the_context.template.engine = DummyEngine()
                content = node.template.render(the_context)

            elif isinstance(node, template.loader_tags.BlockNode):
                safe_context_var = Context(safe_context)
                safe_context_var.render_context['block_context'] = context.render_context.get('block_context', None)
                safe_context_var.template = getattr(node, 'template', None) or template.Template("")
                safe_context_var.template.engine = DummyEngine()
                content = node.render(safe_context_var)

            elif isinstance(node, VariableNode):
                if node.filter_expression.filters:
                    content = node.render(Context(context))
                else:
                    the_context = Context(safe_context)
                    if DJANGO_VERSION >= (1, 8, 0):
                        the_context.template = getattr(node, 'template', None) or template.Template("")
                        the_context.template.engine = DummyEngine()
                    content = node.render(the_context)
            else:
                if DJANGO_VERSION >= (1, 8, 0):
                    # monkey patching for django 1.8
                    the_context = Context(inner_context)
                    the_context.template = getattr(node, 'template', None) or template.Template("")
                    the_context.template.engine = DummyEngine()
                else:
                    the_context = Context(inner_context)
                content = node.render(the_context)

            nodes_content += content
        return nodes_content

    def _get_obj(self, context):
        """return the edited object if exists"""
        return context.get(self.var_name, None) if self.var_name else None

    def _make_inner_context(self, context):
        """the context used for rendering the templatetag content"""
        inner_context = {}
        for ctx_value in context.dicts:
            inner_context.update(ctx_value)

        obj = self._get_obj(context)
        if self.var_name:
            inner_context[self.var_name] = obj

        formset = context.get('formset', None)
        objects = context.get('objects', None)

        if formset:
            inner_context['formset'] = formset
        if objects is not None:
            inner_context['objects'] = objects

        return inner_context

    def _make_outer_context(self, context):
        """the context used for rendering the whole page"""
        obj = self._get_obj(context)
        self.post_url = obj.get_edit_url() if obj else context.get('coop_cms_edit_url')
        outer_context = {'post_url': self.post_url}
        return outer_context

    def render(self, context):
        """to html"""
        request = context.get('request')

        if self._render_logo_size:
            self._logo_size = context.get(self._logo_size, None)

        if self._render_logo_crop:
            self._logo_crop = context.get(self._logo_crop, None)

        # the context used for rendering the templatetag content
        inner_context = self._make_inner_context(context)

        # the context used for rendering the whole page
        outer_context = self._make_outer_context(context)

        # copy of the inner_context to be modified
        safe_context = inner_context.copy()

        form = context.get('form', None)
        obj = self._get_obj(context)

        formset = context.get('formset', None)
        objects = context.get('objects', None)

        if form or formset:
            node_template = template.Template(CMS_FORM_TEMPLATE)
            if form:
                safe_context[self.var_name] = FormWrapper(
                    form, obj, logo_size=self._logo_size, logo_crop=self._logo_crop
                )
            else:
                safe_context['objects'] = [
                    FormWrapper(form_, obj_, logo_size=self._logo_size, logo_crop=self._logo_crop)
                    for (form_, obj_) in zip(formset, objects)
                ]
            outer_context.update(csrf(request))
        else:
            node_template = template.Template("{{inner|safe}}")
            if obj:
                safe_context[self.var_name] = SafeWrapper(
                    obj, logo_size=self._logo_size, logo_crop=self._logo_crop)
            else:
                safe_context['objects'] = [
                    SafeWrapper(obj_, logo_size=self._logo_size, logo_crop=self._logo_crop) for obj_ in objects
                ]

        inner_value = self._render_nodes(context, inner_context, safe_context)

        outer_context['inner'] = mark_safe(inner_value) if (form or formset) else inner_value

        return node_template.render(Context(outer_context))


@register.tag
def cms_edit(parser, token):
    """template tag"""
    args = token.split_contents()[1:]
    data = {}
    var_name = args[0] if len(args) else ''
    for arg in args[1:]:
        key, value = arg.split('=')
        data[key] = value
    nodelist = parser.parse(('end_cms_edit', ))
    parser.next_token()
    return CmsEditNode(nodelist, var_name, **data)


class CmsNoSpace(template.Node):
    """remove space"""
    def __init__(self, nodelist):
        self.nodelist = nodelist

    def render(self, context):
        html = self.nodelist.render(context).strip()
        return ' '.join(html.split())


@register.tag
def cms_nospace(parser, token):
    """remove spaces"""
    nodelist = parser.parse(('end_cms_nospace', ))
    parser.delete_first_token()
    return CmsNoSpace(nodelist)



# -*- coding: utf-8 -*-
"""
views
"""

from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _

from coop_cms.apps.rss_sync.models import RssSource, RssItem
from coop_cms.apps.rss_sync.utils import collect_rss_items, create_cms_article


def collect_rss_items_view(request, source_id):
    """The view called when clicking on the button in the object admin form"""
    rss_source = get_object_or_404(RssSource, id=source_id)

    collect_rss_items(request.user, rss_source)

    url = reverse('admin:rss_sync_rssitem_changelist')+u'?source__id__exact={0}'.format(rss_source.id)
    return HttpResponseRedirect(url)


def collect_rss_items_action(modeladmin, request, queryset):
    """The action called when executed from admin list of rss sources"""
    for source in queryset:
        collect_rss_items(request.user, source)
    url = reverse('admin:rss_sync_rssitem_changelist')
    return HttpResponseRedirect(url)
collect_rss_items_action.short_description = _(u'Collect RSS items')


def create_cms_article_view(request, item_id):
    """The view called when clicking on the button in admin object form"""
    item = get_object_or_404(RssItem, id=item_id)
    art = create_cms_article(request.user, item)
    return HttpResponseRedirect(art.get_edit_url()) #redirect to cms article edit page


def create_cms_article_action(modeladmin, request, queryset):
    """The action called when executed from admin list of rss items"""
    for item in queryset:
        art = create_cms_article(request.user, item)

    #if only 1 item processed (checked)
    if queryset.count() == 1:
        return HttpResponseRedirect(art.get_edit_url()) #redirect to cms article edit page

create_cms_article_action.short_description = _(u'Create CMS Article')



# -*- coding: utf-8 -*-
"""urls"""

from django.conf.urls import include, url
from django.contrib.auth.views import login, password_change, password_reset, password_reset_confirm

from coop_cms.apps.email_auth.forms import BsPasswordChangeForm, BsPasswordResetForm, EmailAuthForm, BsSetPasswordForm


urlpatterns = [
    url(
        r'^login/$',
        login,
        {'authentication_form': EmailAuthForm},
        name='login'
    ),
    url(r'^password_change/$',
        password_change,
        {'password_change_form': BsPasswordChangeForm},
        name='password_change'
    ),
    url(
        r'^password_reset/$',
        password_reset,
        {'password_reset_form': BsPasswordResetForm},
        name='password_reset'
    ),
    url(
        r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        password_reset_confirm,
        {'set_password_form': BsSetPasswordForm},
        name='password_reset_confirm'
    ),
    url(r'^', include('django.contrib.auth.urls')),
]



# -*- coding: utf-8 -*-
"""widgets"""

from floppyforms.widgets import ClearableFileInput, Select, SelectMultiple, Input

from coop_cms.utils import get_text_from_template


class ReadOnlyInput(Input):
    """readonly input"""
    template_name = 'coop_cms/widgets/readonlyinput.html'


class ImageEdit(ClearableFileInput):
    """image edit"""
    template_name = 'coop_cms/widgets/imageedit.html'
    
    def __init__(self, update_url, thumbnail_src, *args, **kwargs):
        super(ImageEdit, self).__init__(*args, **kwargs)
        self._extra_context = {
            'update_url': update_url,
            'thumbnail_src': thumbnail_src,
            'extra_classes': get_text_from_template("coop_cms/widgets/_imageedit_cssclass.html"),
        }
        
    def get_context(self, *args, **kwargs):
        """get context"""
        context = super(ImageEdit, self).get_context(*args, **kwargs)
        context.update(self._extra_context)
        return context


class ChosenWidgetMixin(object):
    """chosen jquery widget"""

    def _patch(self, kwargs):

        self._extra_context = {}
        if kwargs.pop("force_template", False):
            # chosen inherit from super template
            self._extra_context['super_template'] = self.template_name
            self.template_name = 'coop_cms/widgets/chosen.html'

        self._extra_context['on_popup'] = kwargs.pop("on_popup", False)

        return kwargs


class ChosenSelectMultiple(ChosenWidgetMixin, SelectMultiple):
    """chosen select multiple"""

    def __init__(self, attrs=None, *args, **kwargs):

        kwargs = self._patch(kwargs)

        if not attrs:
            attrs = {}
        attrs['data-placeholder'] = kwargs.pop('overlay', None)
        super(ChosenSelectMultiple, self).__init__(attrs, *args, **kwargs)

    def get_context(self, *args, **kwargs):
        """context"""
        context = super(ChosenSelectMultiple, self).get_context(*args, **kwargs)  # pylint: disable=E1002
        context.update(self._extra_context)
        return context

    class Media:
        """css and js required by widget"""
        js = (
            "{0}?v=1".format("chosen/chosen.jquery.min.js"),
        )
        css = {
            "all": ("{0}?v=1".format("chosen/chosen.css"),),
        }


class ChosenSelect(ChosenWidgetMixin, Select):
    """chosen select"""

    def __init__(self, attrs=None, *args, **kwargs):
        kwargs = self._patch(kwargs)

        if not attrs:
            attrs = {}
        attrs['data-placeholder'] = kwargs.pop('overlay', None)
        super(ChosenSelect, self).__init__(attrs, *args, **kwargs)

    def get_context(self, *args, **kwargs):
        """context"""
        context = super(ChosenSelect, self).get_context(*args, **kwargs)  # pylint: disable=E1002
        context.update(self._extra_context)
        return context

    class Media:
        """css and js required by widget"""
        js = (
            "{0}?v=1".format("chosen/chosen.jquery.min.js"),
        )
        css = {
            "all": ("{0}?v=1".format("chosen/chosen.css"),),
        }


# -*- coding: utf-8 -*-
"""send newsletter"""

from datetime import datetime

from django.core.management.base import BaseCommand

from coop_cms.utils import send_newsletter
from coop_cms.models import NewsletterSending


class Command(BaseCommand):
    """send newsletter"""
    help = u"send newsletter"
    use_argparse = False

    def handle(self, email_addresses, *args, **options):
        """command"""
        #look for emailing to be sent

        verbose = options.get('verbosity', 1)
        
        if email_addresses:
            email_list = email_addresses.split(";")
        else:
            print 'usage: python manage.py send_newsletter toto@toto.fr;titi@titi.fr'
        
        sendings = NewsletterSending.objects.filter(scheduling_dt__lte=datetime.now(), sending_dt=None)
        for sending in sendings:
            if verbose:
                print 'send_newsletter {1} to {0} addresses'.format(len(email_list), sending.newsletter)
            
            nb_sent = send_newsletter(sending.newsletter, email_list)
            
            if verbose:
                print nb_sent, "emails sent"
            
            sending.sending_dt = datetime.now()
            sending.save()



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'NavType.content_type'
        db.alter_column(u'coop_cms_navtype', 'content_type_id', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['contenttypes.ContentType'], unique=True))

    def backwards(self, orm):

        # Changing field 'NavType.content_type'
        db.alter_column(u'coop_cms_navtype', 'content_type_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], unique=True))

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'coop_cms.alias': {
            'Meta': {'object_name': 'Alias'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'path': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'redirect_url': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        u'coop_cms.articlecategory': {
            'Meta': {'object_name': 'ArticleCategory'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_rss': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'default': '[1]', 'to': u"orm['sites.Site']", 'symmetrical': 'False'}),
            'slug': ('django_extensions.db.fields.AutoSlugField', [], {'allow_duplicates': 'False', 'max_length': '100', 'separator': "u'-'", 'blank': 'True', 'unique': 'True', 'populate_from': "'name'", 'overwrite': 'False'})
        },
        u'coop_cms.document': {
            'Meta': {'object_name': 'Document'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': u"orm['coop_cms.ArticleCategory']", 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'filters': ('django.db.models.fields.related.ManyToManyField', [], {'default': 'None', 'to': u"orm['coop_cms.MediaFilter']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_private': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '100'})
        },
        u'coop_cms.fragment': {
            'Meta': {'ordering': "('position', 'id')", 'object_name': 'Fragment'},
            'content': ('django.db.models.fields.TextField', [], {'default': "u''", 'blank': 'True'}),
            'css_class': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '100', 'blank': 'True'}),
            'filter': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': u"orm['coop_cms.FragmentFilter']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'position': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['coop_cms.FragmentType']"})
        },
        u'coop_cms.fragmentfilter': {
            'Meta': {'object_name': 'FragmentFilter'},
            'extra_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'coop_cms.fragmenttype': {
            'Meta': {'object_name': 'FragmentType'},
            'allowed_css_classes': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'})
        },
        u'coop_cms.image': {
            'Meta': {'object_name': 'Image'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'filters': ('django.db.models.fields.related.ManyToManyField', [], {'default': 'None', 'to': u"orm['coop_cms.MediaFilter']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '100'}),
            'size': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': u"orm['coop_cms.ImageSize']", 'null': 'True', 'blank': 'True'})
        },
        u'coop_cms.imagesize': {
            'Meta': {'object_name': 'ImageSize'},
            'crop': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '100', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'size': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'coop_cms.link': {
            'Meta': {'object_name': 'Link'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'default': "u'title'", 'max_length': '200'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        u'coop_cms.mediafilter': {
            'Meta': {'object_name': 'MediaFilter'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'coop_cms.navnode': {
            'Meta': {'object_name': 'NavNode'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_navigation': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'ordering': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'default': '0', 'to': u"orm['coop_cms.NavNode']", 'null': 'True', 'blank': 'True'}),
            'tree': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['coop_cms.NavTree']"})
        },
        u'coop_cms.navtree': {
            'Meta': {'object_name': 'NavTree'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'default'", 'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'types': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'coop_cms_navtree_set'", 'blank': 'True', 'to': u"orm['coop_cms.NavType']"})
        },
        u'coop_cms.navtype': {
            'Meta': {'object_name': 'NavType'},
            'content_type': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['contenttypes.ContentType']", 'unique': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label_rule': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'search_field': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        u'coop_cms.newsletter': {
            'Meta': {'object_name': 'Newsletter'},
            'content': ('django.db.models.fields.TextField', [], {'default': "'<br>'", 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'items': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['coop_cms.NewsletterItem']", 'symmetrical': 'False', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'source_url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'subject': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'template': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        u'coop_cms.newsletteritem': {
            'Meta': {'ordering': "['ordering']", 'unique_together': "(('content_type', 'object_id'),)", 'object_name': 'NewsletterItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        u'coop_cms.newslettersending': {
            'Meta': {'object_name': 'NewsletterSending'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'newsletter': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['coop_cms.Newsletter']"}),
            'scheduling_dt': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'sending_dt': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'})
        },
        u'coop_cms.pieceofhtml': {
            'Meta': {'object_name': 'PieceOfHtml'},
            'content': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'div_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'extra_id': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '100', 'db_index': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'coop_cms.sitesettings': {
            'Meta': {'ordering': "('site__id',)", 'object_name': 'SiteSettings'},
            'homepage_url': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '256', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['sites.Site']", 'unique': 'True'}),
            'sitemap_mode': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        u'sites.site': {
            'Meta': {'ordering': "(u'domain',)", 'object_name': 'Site', 'db_table': "u'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['coop_cms']


# -*- coding: utf-8 -*-

from django.conf import settings

from coop_cms.models import BaseArticle
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseArticleTest
from coop_cms.utils import get_login_url


class PermissionMiddlewareTest(BaseArticleTest):
    
    def setUp(self):
        super(PermissionMiddlewareTest, self).setUp()
        self._MIDDLEWARE_CLASSES = settings.MIDDLEWARE_CLASSES
        if not 'coop_cms.middleware.PermissionsMiddleware' in settings.MIDDLEWARE_CLASSES:
            settings.MIDDLEWARE_CLASSES += ('coop_cms.middleware.PermissionsMiddleware',)
        
    def tearDown(self):
        super(PermissionMiddlewareTest, self).tearDown()
        self.MIDDLEWARE_CLASSES = self._MIDDLEWARE_CLASSES
        
    def test_view_draft_anonymous(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.DRAFT)
        self.assertEqual(article.is_draft(), True)
        url = article.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        auth_url = get_login_url()
        self.assertRedirects(response, auth_url+'?next='+url)
        
    def test_edit_anonymous(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.DRAFT)
        self.assertEqual(article.is_draft(), True)
        url = article.get_edit_url()
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        auth_url = get_login_url()
        self.assertTrue(response["Location"].find(auth_url+'?next='+url) >= 0)
        
    def test_view_published_anonymous(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        self.assertEqual(article.is_draft(), False)
        url = article.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
    def test_view_draft_not_allowed(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.DRAFT)
        self.assertEqual(article.is_draft(), True)
        
        self._log_as_non_editor()
        
        url = article.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)



# -*- coding: utf-8 -*-

from datetime import timedelta
from unittest import skipIf

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core import mail
from django.core import management
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.test.utils import override_settings
from django.utils import timezone
from django.utils.translation import ugettext

from model_mommy import mommy

from coop_cms.models import Newsletter, NewsletterItem, PieceOfHtml, NewsletterSending
from coop_cms.settings import is_localized, get_article_class
from coop_cms.tests import BaseTestCase, UserBaseTestCase, BeautifulSoup
from coop_cms.utils import make_links_absolute, strip_a_tags, avoid_line_too_long, send_email


@override_settings(COOP_CMS_NEWSLETTER_SETTINGS_FORM='')
class NewsletterSettingsTest(UserBaseTestCase):

    def setUp(self):
        super(NewsletterSettingsTest, self).setUp()
        self._COOP_CMS_NEWSLETTER_TEMPLATES = getattr(settings, 'COOP_CMS_NEWSLETTER_TEMPLATES', None)
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_blue.html', 'Blue'),
            ('test/newsletter_red.html', 'Blue'),
        )

    def tearDown(self):
        super(NewsletterSettingsTest, self).tearDown()
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = self._COOP_CMS_NEWSLETTER_TEMPLATES

    def test_view_create_newsletter(self):
        self._log_as_editor()
        url = reverse("coop_cms_new_newsletter")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Newsletter.objects.count(), 0)

    def test_create_newsletter(self):
        self._log_as_editor()
        url = reverse("coop_cms_new_newsletter")

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': []
        }

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Newsletter.objects.count(), 1)
        newsletter = Newsletter.objects.all()[0]

        self.assertEqual(newsletter.subject, data["subject"])
        self.assertEqual(newsletter.template, data["template"])

    def test_view_edit_newsletter(self):
        self._log_as_editor()
        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_blue.html",
        )

        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_edit_newsletter(self):
        self._log_as_editor()
        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_red.html",
        )
        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': []
        }

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Newsletter.objects.count(), 1)
        newsletter = Newsletter.objects.all()[0]

        self.assertEqual(newsletter.subject, data["subject"])
        self.assertEqual(newsletter.template, data["template"])

    def test_view_newsletter_items(self):
        self._log_as_editor()
        article_class = get_article_class()

        site = Site.objects.get_current()
        other_site = mommy.make(Site)

        article_1 = mommy.make(article_class, slug="test1", in_newsletter=True)
        article_2 = mommy.make(article_class, slug="test2", in_newsletter=True, sites=[other_site])
        article_3 = mommy.make(article_class, slug="test3", in_newsletter=True)
        article_4 = mommy.make(article_class, slug="test4", in_newsletter=False)

        article_2.sites.clear()
        article_2.sites.add(other_site)
        article_2.save()

        ct = ContentType.objects.get_for_model(article_class)
        self.assertEqual(3, NewsletterItem.objects.count())
        item_1 = NewsletterItem.objects.get(content_type=ct, object_id=article_1.id)
        item_2 = NewsletterItem.objects.get(content_type=ct, object_id=article_2.id)
        item_3 = NewsletterItem.objects.get(content_type=ct, object_id=article_3.id)

        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_red.html",
            items=[item_1]
        )

        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        options = soup.select("#id_items option")
        self.assertEqual(2, len(options))
        self.assertEqual(
            sorted([int(id_["value"]) for id_ in options]),
            sorted([item.id for item in [item_1, item_3]])
        )

        options = soup.select("#id_items option[selected=selected]")
        self.assertEqual(1, len(options))
        self.assertEqual(
            sorted([int(id_["value"]) for id_ in options]),
            sorted([item.id for item in [item_1]])
        )

    def test_save_newsletter_items(self):
        self._log_as_editor()
        article_class = get_article_class()

        site = Site.objects.get_current()
        other_site = mommy.make(Site)

        article_1 = mommy.make(article_class, slug="test1", in_newsletter=True)
        article_2 = mommy.make(article_class, slug="test2", in_newsletter=True)
        article_3 = mommy.make(article_class, slug="test3", in_newsletter=True)

        ct = ContentType.objects.get_for_model(article_class)
        self.assertEqual(3, NewsletterItem.objects.count())
        item_1 = NewsletterItem.objects.get(content_type=ct, object_id=article_1.id)
        item_2 = NewsletterItem.objects.get(content_type=ct, object_id=article_2.id)
        item_3 = NewsletterItem.objects.get(content_type=ct, object_id=article_3.id)

        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_red.html",
            items=[item_1]
        )

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': [item_2.id, item_3.id]
        }

        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)

        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.subject, data["subject"])
        self.assertEqual(newsletter.template, data["template"])
        self.assertEqual(
            list(sorted(n.id for n in newsletter.items.all())),
            list(sorted(data["items"]))
        )

    def test_save_newsletter_items_other_site(self):
        self._log_as_editor()
        article_class = get_article_class()

        site = Site.objects.get_current()
        other_site = mommy.make(Site)

        article_1 = mommy.make(article_class, slug="test1", in_newsletter=True)
        article_2 = mommy.make(article_class, slug="test2", in_newsletter=True)
        article_3 = mommy.make(article_class, slug="test3", in_newsletter=True)

        article_2.sites.clear()
        article_2.sites.add(other_site)
        article_2.save()

        content_type = ContentType.objects.get_for_model(article_class)
        self.assertEqual(3, NewsletterItem.objects.count())
        item_1 = NewsletterItem.objects.get(content_type=content_type, object_id=article_1.id)
        item_2 = NewsletterItem.objects.get(content_type=content_type, object_id=article_2.id)
        item_3 = NewsletterItem.objects.get(content_type=content_type, object_id=article_3.id)

        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_red.html",
            items=[item_1]
        )

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': [item_2.id, item_3.id]
        }

        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        self.assertEqual(len(soup.select(".errorlist")), 1)
        self.assertEqual(len(soup.select("#id_items")[0].parent.select(".errorlist")), 1)

        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertNotEqual(newsletter.subject, data["subject"])
        self.assertNotEqual(newsletter.template, data["template"])
        self.assertEqual([item_1.id], [x.id for x in newsletter.items.all()])

    def test_save_newsletter_items_no_newsletter(self):
        self._log_as_editor()
        article_class = get_article_class()

        site = Site.objects.get_current()
        other_site = mommy.make(Site)

        article_1 = mommy.make(article_class, slug="test1", in_newsletter=True)
        article_2 = mommy.make(article_class, slug="test2", in_newsletter=True)
        article_3 = mommy.make(article_class, slug="test3", in_newsletter=True)

        content_type = ContentType.objects.get_for_model(article_class)
        self.assertEqual(3, NewsletterItem.objects.count())
        item_1 = NewsletterItem.objects.get(content_type=content_type, object_id=article_1.id)
        item_2 = NewsletterItem.objects.get(content_type=content_type, object_id=article_2.id)
        item_3 = NewsletterItem.objects.get(content_type=content_type, object_id=article_3.id)

        article_2.in_newsletter = False
        article_2.save()

        self.assertEqual(2, NewsletterItem.objects.count())

        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_red.html",
            items=[item_1]
        )

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': [item_2.id, item_3.id]
        }

        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        self.assertEqual(len(soup.select(".errorlist")), 1)
        self.assertEqual(len(soup.select("#id_items")[0].parent.select(".errorlist")), 1)

        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertNotEqual(newsletter.subject, data["subject"])
        self.assertNotEqual(newsletter.template, data["template"])
        self.assertEqual([item_1.id], [x.id for x in newsletter.items.all()])

    def test_view_create_newsletter_not_logged(self):
        url = reverse("coop_cms_new_newsletter")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)

    def test_create_newsletter_not_logged(self):
        url = reverse("coop_cms_new_newsletter")

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': []
        }

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Newsletter.objects.count(), 0)

    def test_view_edit_newsletter_not_logged(self):
        newsletter = mommy.make(
            Newsletter,
            subject="a little intro for this newsletter",
            template="test/newsletter_blue.html",
        )

        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)

    def test_edit_newsletter_not_logged(self):
        subject = "a little intro for this newsletter"
        newsletter = mommy.make(
            Newsletter,
            subject=subject,
            template="test/newsletter_red.html",
        )
        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])

        data = {
            "subject": "test",
            "template": "test/newsletter_blue.html",
            'items': []
        }

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Newsletter.objects.count(), 1)
        newsletter = Newsletter.objects.all()[0]

        self.assertEqual(newsletter.subject, subject)


class NewsletterTest(UserBaseTestCase):

    def test_create_article_for_newsletter(self):
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        
        art = mommy.make(article_class, slug="test", in_newsletter=True)
        
        self.assertEqual(1, NewsletterItem.objects.count())
        item = NewsletterItem.objects.get(content_type=content_type, object_id=art.id)
        self.assertEqual(item.content_object, art)
        
        art.delete()
        self.assertEqual(0, NewsletterItem.objects.count())

    def test_create_article_not_for_newsletter(self):
        article_class = get_article_class()
        ContentType.objects.get_for_model(article_class)
        
        art = mommy.make(article_class, slug="test", in_newsletter=False)
        self.assertEqual(0, NewsletterItem.objects.count())
        
        art.delete()
        self.assertEqual(0, NewsletterItem.objects.count())

    def test_create_article_commands(self):
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        art1 = mommy.make(article_class, slug="test1", in_newsletter=True)
        art2 = mommy.make(article_class, slug="test2", in_newsletter=True)
        art3 = mommy.make(article_class, slug="test3", in_newsletter=False)
        self.assertEqual(2, NewsletterItem.objects.count())
        NewsletterItem.objects.all().delete()
        self.assertEqual(0, NewsletterItem.objects.count())
        management.call_command('create_newsletter_items', verbosity=0, interactive=False)
        self.assertEqual(2, NewsletterItem.objects.count())
        item1 = NewsletterItem.objects.get(content_type=content_type, object_id=art1.id)
        item2 = NewsletterItem.objects.get(content_type=content_type, object_id=art2.id)

    def test_view_newsletter(self):
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        
        art1 = mommy.make(article_class, title="Art 1", in_newsletter=True)
        art2 = mommy.make(article_class, title="Art 2", in_newsletter=True)
        art3 = mommy.make(article_class, title="Art 3", in_newsletter=True)
        
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
            is_public=True
        )
        newsletter.items.add(NewsletterItem.objects.get(content_type=content_type, object_id=art1.id))
        newsletter.items.add(NewsletterItem.objects.get(content_type=content_type, object_id=art2.id))
        newsletter.save()
        
        url = reverse('coop_cms_view_newsletter', args=[newsletter.id])
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
        self.assertContains(response, newsletter.content)
        self.assertContains(response, art1.title)
        self.assertContains(response, art2.title)
        self.assertNotContains(response, art3.title)

    def test_view_newsletter_not_public(self):

        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
        )

        url = reverse('coop_cms_view_newsletter', args=[newsletter.id])
        response = self.client.get(url)

        self.assertEqual(403, response.status_code)
        
    def test_edit_newsletter_is_public(self):
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
            is_public=True
        )

        self._log_as_editor()
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
        data = {'content': 'A better intro'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        self.assertNotContains(response, newsletter.content)
        self.assertContains(response, data['content'])

    def test_edit_newsletter_not_public(self):
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
            is_public=False
        )

        self._log_as_editor()
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)

        data = {'content': 'A better intro'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, newsletter.content)
        self.assertContains(response, data['content'])

    def test_edit_newsletter_anonymous(self):
        original_data = {
            'content': "a little intro for this newsletter",
            'template': "test/newsletter_blue.html"
        }
        newsletter = mommy.make(Newsletter, **original_data)
        
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)
        
        response = self.client.post(url, data={'content': ':OP'})
        self.assertEqual(403, response.status_code)
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.content, original_data['content'])
        
    def test_edit_newsletter_no_articles(self):
        self._log_as_editor()
        original_data = {
            'content': "a little intro for this newsletter",
            'template': "test/newsletter_blue.html"
        }
        newsletter = mommy.make(Newsletter, **original_data)
        
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        
        data = {'content': ':OP'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, data['content'])
        
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, data['content'])
        
    def test_newsletter_templates(self):
        """it should send email with right content"""
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        
        art1 = mommy.make(article_class, title="Art 1", in_newsletter=True)
        poh = mommy.make(PieceOfHtml, div_id="newsletter_header", content="HELLO!!!")
        
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html"
        )
        newsletter.items.add(NewsletterItem.objects.get(content_type=content_type, object_id=art1.id))
        newsletter.save()
        
        self._log_as_editor()
        
        view_names = ['coop_cms_view_newsletter', 'coop_cms_edit_newsletter']
        for view_name in view_names:

            url = reverse(view_name, args=[newsletter.id])
            response = self.client.get(url)
            self.assertEqual(200, response.status_code)

            self.assertContains(response, newsletter.content)
            self.assertContains(response, art1.title)
            self.assertContains(response, "background: blue;")
            self.assertNotContains(response, poh.content)
        
        newsletter.template = "test/newsletter_red.html"
        newsletter.save()
        
        for view_name in view_names:
            url = reverse(view_name, args=[newsletter.id])
            response = self.client.get(url)
            
            self.assertEqual(200, response.status_code)
            
            self.assertContains(response, newsletter.content)
            self.assertContains(response, art1.title)
            self.assertContains(response, "background: red;")
            self.assertContains(response, poh.content)
            
    def test_change_newsletter_templates(self):
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        self._log_as_editor()
        
        newsletter = mommy.make(Newsletter, template='test/newsletter_blue.html')
        
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
        for tpl, name in settings.COOP_CMS_NEWSLETTER_TEMPLATES:
            self.assertContains(response, tpl)
            self.assertContains(response, name)
            
        data={'template': 'test/newsletter_red.html'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, reverse('coop_cms_edit_newsletter', args=[newsletter.id]))
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.template, data['template'])
        
    def test_change_newsletter_templates_anonymous(self):
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        original_data={'template': 'test/newsletter_blue.html'}
        newsletter = mommy.make(Newsletter, **original_data)
        
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        
        data={'template': 'test/newsletter_red.html'}
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.template, original_data['template'])
        
    def test_change_newsletter_unknow_template(self):
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        original_data={'template': 'test/newsletter_blue.html'}
        newsletter = mommy.make(Newsletter, **original_data)
        
        self._log_as_editor()
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        data={'template': 'test/newsletter_yellow.html'}
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.template, original_data['template'])

    @override_settings(COOP_CMS_TEST_EMAILS=['toto@toto.fr', 'titi@toto.fr'], COOP_CMS_FROM_EMAIL='contact@toto.fr')
    def test_send_test_newsletter(self, template='test/newsletter_blue.html', extra_checker=None):
        settings.SITE_ID = 1
        site = Site.objects.get(id=settings.SITE_ID)
        site.domain = 'toto.fr'
        site.save()
        
        rel_content = u'''
            <h1>Title</h1><a href="{0}/toto/"><img src="{0}/toto.jpg"></a><br /><img src="{0}/toto.jpg">
            <div><a href="http://www.google.fr">Google</a></div>
        '''
        original_data = {
            'template': template,
            'subject': 'test email',
            'content': rel_content.format("")
        }
        newsletter = mommy.make(Newsletter, **original_data)
        
        self._log_as_editor()
        url = reverse('coop_cms_test_newsletter', args=[newsletter.id])
        response = self.client.post(url, data={})
        self.assertEqual(200, response.status_code)
        
        self.assertEqual(
            [[test_address] for test_address in settings.COOP_CMS_TEST_EMAILS],
            [received_email.to for received_email in mail.outbox]
        )
        for received_email in mail.outbox:
            self.assertEqual(received_email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(received_email.subject, newsletter.subject)
            self.assertTrue(received_email.body.find('Title') >= 0)
            self.assertTrue(received_email.body.find('Google') >= 0)
            self.assertTrue(received_email.alternatives[0][1], "text/html")
            self.assertTrue(received_email.alternatives[0][0].find('Title') >= 0)
            self.assertTrue(received_email.alternatives[0][0].find('Google') >= 0)
            site_prefix = "http://"+site.domain
            self.assertTrue(received_email.alternatives[0][0].find(site_prefix) >= 0)
            if extra_checker:
                extra_checker(received_email)
        
    def test_schedule_newsletter_sending(self):
        newsletter = mommy.make(Newsletter)
        
        self._log_as_editor()
        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
        
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
        sch_dt = "2030-12-12 12:00:00"
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertContains(response, '$.colorbox.close()')
        self.assertEqual(1, NewsletterSending.objects.count())
        self.assertEqual(newsletter, NewsletterSending.objects.all()[0].newsletter)
        self.assertEqual(2030, NewsletterSending.objects.all()[0].scheduling_dt.year)
        
    def test_schedule_newsletter_sending_invalid_value(self):
        newsletter = mommy.make(Newsletter)
        
        self._log_as_editor()
        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
        
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
        sch_dt = ''
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, NewsletterSending.objects.count())
        
        sch_dt = 'toto'
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, NewsletterSending.objects.count())
        
        sch_dt = "2005-12-12 12:00:00"
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, NewsletterSending.objects.count())
    
    def test_schedule_anonymous(self):
        newsletter = mommy.make(Newsletter)
        
        login_url = reverse('django.contrib.auth.views.login')
        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
        
        response = self.client.get(url, follow=False)
        redirect_url = response['Location']
        if is_localized():
            login_url = login_url[:2]
            self.assertTrue(redirect_url.find(login_url) >= 0)
        else:
            self.assertTrue(redirect_url.find(login_url) >= 0)
        
        sch_dt = timezone.now()+timedelta(1)
        response = self.client.post(url, data={'sending_dt': sch_dt})
        redirect_url = response['Location']
        self.assertTrue(redirect_url.find(login_url) >= 0)
    
    def test_send_newsletter(self):
        
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello guys!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_blue.html',
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        self.assertEqual(newsletter.is_public, False)
        sch_dt = timezone.now() - timedelta(1)
        sending = mommy.make(NewsletterSending, newsletter=newsletter, scheduling_dt= sch_dt, sending_dt= None)

        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)

        sending = NewsletterSending.objects.get(id=sending.id)
        self.assertNotEqual(sending.sending_dt, None)
        
        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, ['toto@toto.fr'])
        self.assertEqual(email.subject, newsletter_data['subject'])
        self.assertTrue(email.body.find('Hello guys') >= 0)
        self.assertTrue(email.alternatives[0][1], "text/html")
        self.assertTrue(email.alternatives[0][0].find('Hello guys') >= 0)

        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.is_public, True)
        
        # check whet happens if command is called again
        mail.outbox = []
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        self.assertEqual(len(mail.outbox), 0)

    def test_send_newsletter_several(self):
        """it should send newsletter to several addresses"""

        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello guys!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_blue.html',
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        
        sch_dt = timezone.now() - timedelta(1)
        sending = mommy.make(NewsletterSending, newsletter=newsletter, scheduling_dt= sch_dt, sending_dt= None)
        
        addresses = ';'.join(['toto@toto.fr']*5)
        management.call_command('send_newsletter', addresses, verbosity=0, interactive=False)
        
        sending = NewsletterSending.objects.get(id=sending.id)
        self.assertNotEqual(sending.sending_dt, None)
        
        self.assertEqual(len(mail.outbox), 5)
        for email in mail.outbox:
            self.assertEqual(email.to, ['toto@toto.fr'])
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find('Hello guys')>=0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find('Hello guys')>=0)
        
        # check whet happens if command is called again
        mail.outbox = []
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        self.assertEqual(len(mail.outbox), 0)

    def test_send_newsletter_not_yet(self):
        
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello guys!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_blue.html',
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        
        sch_dt = timezone.now() + timedelta(1)
        sending = mommy.make(NewsletterSending, newsletter=newsletter, scheduling_dt= sch_dt, sending_dt= None)
        
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        
        sending = NewsletterSending.objects.get(id=sending.id)
        self.assertEqual(sending.sending_dt, None)
        
        self.assertEqual(len(mail.outbox), 0)


@override_settings(COOP_CMS_FROM_EMAIL='contact@toto.fr')
class SendEmailTest(UserBaseTestCase):
    """test utils.send_email"""

    def test_send_email(self):
        settings.SITE_ID = 1
        site = Site.objects.get(id=settings.SITE_ID)
        site.domain = 'toto.fr'
        site.save()

        site2 = mommy.make(Site, domain="toto.com")

        # subject, template_name, context, site_domain, dests, lang = None, list_unsubscribe = None

        subject = "Hello"
        dests = ['toto@toto.fr']

        send_email(subject, "test/test_send_email.html", {'value': 'ABCD'}, "http://" + site2.domain, dests)

        self.assertEqual(
            [[test_address] for test_address in dests],
            [received_email.to for received_email in mail.outbox]
        )
        for received_email in mail.outbox:
            self.assertEqual(received_email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(received_email.subject, subject)
            self.assertTrue(received_email.body.find('ABCD') >= 0)
            self.assertTrue(received_email.alternatives[0][1], "text/html")
            self.assertTrue(received_email.alternatives[0][0].find('ABCD') >= 0)
            site_prefix = "http://" + site2.domain
            self.assertTrue(received_email.alternatives[0][0].find(site_prefix) >= 0)

    @skipIf('en' not in [lang_code for (lang_code, lang_name) in settings.LANGUAGES], 'English required for this test')
    def test_send_email_en(self):
        settings.SITE_ID = 1
        site = Site.objects.get(id=settings.SITE_ID)
        site.domain = 'toto.fr'
        site.save()

        site2 = mommy.make(Site, domain="toto.com")

        # subject, template_name, context, site_domain, dests, lang = None, list_unsubscribe = None

        subject = "Hello"
        dests = ['toto@toto.fr']

        send_email(subject, "test/test_send_email.html", {'value': 'ABCD'}, "http://" + site2.domain, dests, 'en')

        self.assertEqual(
            [[test_address] for test_address in dests],
            [received_email.to for received_email in mail.outbox]
        )
        for received_email in mail.outbox:

            translated = 'All'  # ugettext('All')

            self.assertEqual(received_email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(received_email.subject, subject)
            self.assertTrue(received_email.body.find('ABCD') >= 0)
            self.assertTrue(received_email.body.find(translated) >= 0)
            self.assertTrue(received_email.alternatives[0][1], "text/html")
            self.assertTrue(received_email.alternatives[0][0].find('ABCD') >= 0)
            self.assertTrue(received_email.alternatives[0][0].find(translated) >= 0)
            site_prefix = "http://" + site2.domain
            self.assertTrue(received_email.alternatives[0][0].find(site_prefix) >= 0)

    @skipIf('fr' not in [lang_code for (lang_code, lang_name) in settings.LANGUAGES], 'French required for this test')
    def test_send_email_fr(self):
        settings.SITE_ID = 1
        site = Site.objects.get(id=settings.SITE_ID)
        site.domain = 'toto.fr'
        site.save()

        site2 = mommy.make(Site, domain="toto.com")

        # subject, template_name, context, site_domain, dests, lang = None, list_unsubscribe = None

        subject = "Hello"
        dests = ['toto@toto.fr']

        send_email(subject, "test/test_send_email.html", {'value': 'ABCD'}, "http://" + site2.domain, dests, 'fr')

        self.assertEqual(
            [[test_address] for test_address in dests],
            [received_email.to for received_email in mail.outbox]
        )
        for received_email in mail.outbox:
            translated = 'Tous'  # ugettext('All')

            self.assertEqual(received_email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(received_email.subject, subject)
            self.assertTrue(received_email.body.find('ABCD') >= 0)
            self.assertTrue(received_email.body.find(translated) >= 0)
            self.assertTrue(received_email.alternatives[0][1], "text/html")
            self.assertTrue(received_email.alternatives[0][0].find('ABCD') >= 0)
            self.assertTrue(received_email.alternatives[0][0].find(translated) >= 0)
            site_prefix = "http://" + site2.domain
            self.assertTrue(received_email.alternatives[0][0].find(site_prefix) >= 0)

    @skipIf('sn' in [lang_code for (lang_code, lang_name) in settings.LANGUAGES], 'No Chinese for this test')
    def test_send_email_sn(self):
        settings.SITE_ID = 1
        site = Site.objects.get(id=settings.SITE_ID)
        site.domain = 'toto.fr'
        site.save()

        site2 = mommy.make(Site, domain="toto.com")

        # subject, template_name, context, site_domain, dests, lang = None, list_unsubscribe = None

        subject = "Hello"
        dests = ['toto@toto.fr']

        send_email(subject, "test/test_send_email.html", {'value': 'ABCD'}, "http://" + site2.domain, dests, 'sn')

        self.assertEqual(
            [[test_address] for test_address in dests],
            [received_email.to for received_email in mail.outbox]
        )
        for received_email in mail.outbox:
            translated = 'All'  # ugettext('All')

            self.assertEqual(received_email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(received_email.subject, subject)
            self.assertTrue(received_email.body.find('ABCD') >= 0)
            self.assertTrue(received_email.body.find(translated) >= 0)
            self.assertTrue(received_email.alternatives[0][1], "text/html")
            self.assertTrue(received_email.alternatives[0][0].find('ABCD') >= 0)
            self.assertTrue(received_email.alternatives[0][0].find(translated) >= 0)
            site_prefix = "http://" + site2.domain
            self.assertTrue(received_email.alternatives[0][0].find(site_prefix) >= 0)


class AbsUrlTest(UserBaseTestCase):
    
    def setUp(self):
        super(AbsUrlTest, self).setUp()
        settings.SITE_ID = 1
        self.site = Site.objects.get(id=settings.SITE_ID)
        self.site.domain = "toto.fr"
        self.site.save()
        self.site_prefix = "http://"+self.site.domain
        self.newsletter = mommy.make(Newsletter, site=self.site)
        #settings.COOP_CMS_SITE_PREFIX = self.site_prefix

    def _to_text(self, soup):
        text = soup.prettify()
        text = strip_a_tags(text)
        return text
    
    def test_href(self):
        test_html = '<a href="%s/toto">This is a link</a>'
        rel_html = test_html % ""
        abs_html = self._to_text(BeautifulSoup(test_html % self.site_prefix))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(rel_html, self.newsletter)))
        
    def test_src(self):
        test_html = '<h1>My image</h1><img src="%s/toto">'
        rel_html = test_html % ""
        abs_html = self._to_text(BeautifulSoup(test_html % self.site_prefix))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(rel_html, self.newsletter)))
        
    def test_relative_path(self):
        test_html = '<h1>My image</h1><img src="%s/toto">'
        rel_html = test_html % "../../.."
        abs_html = self._to_text(BeautifulSoup(test_html % self.site_prefix))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(rel_html, self.newsletter)))
    
    def test_src_and_img(self):
        test_html = '<h1>My image</h1><a href="{0}/a1">This is a link</a><img src="{0}/toto"/><img src="{0}/titi"/>' + \
            '<a href="{0}/a2">This is another link</a>'
        rel_html = test_html.format("")
        html = test_html.format(self.site_prefix)
        abs_html = self._to_text(BeautifulSoup(html))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(rel_html, self.newsletter)))
        
    def test_href_rel_and_abs(self):
        test_html = '<a href="%s/toto">This is a link</a><a href="http://www.apidev.fr">another</a>'
        rel_html = test_html % ""
        abs_html = self._to_text(BeautifulSoup(test_html % self.site_prefix))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(rel_html, self.newsletter)))
        
    def test_style_in_between(self):
        test_html = u'<img style="margin: 0; width: 700px;" src="%s/media/img/newsletter_header.png" alt="Logo">'
        rel_html = test_html % ""
        abs_html = self._to_text(BeautifulSoup(test_html % self.site_prefix))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(rel_html, self.newsletter)))
        
    def test_missing_attr(self):
        test_html = u'<img alt="Logo" /><a name="aa">link</a>'
        abs_html = self._to_text(BeautifulSoup(test_html))
        self.assertEqual(abs_html, strip_a_tags(make_links_absolute(test_html, self.newsletter)))


class NewsletterFriendlyTemplateTagsTest(BaseTestCase):
    
    template_content = """
        {{% load coop_utils %}}
        {{% nlf_css {0} %}}
            <a>One</a>
            <a>Two</a>
            <a>Three</a>
            <img />
            <table><tr><td></td><td></td></table>
            <table class="this-one"><tr><td></td><td></td></table>
        {{% end_nlf_css %}}
    """
    
    def test_email_mode_is_inline(self):
        template = self.template_content.format('a="color: red;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(0, html.count(u'<style>'))
        self.assertEqual(3, html.count(u'<a style="color: red;">'))
        
    def test_edit_mode_is_in_style(self):
        template = self.template_content.format('a="color: red;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': False}))
        self.assertEqual(1, html.count(u'<style>'))
        self.assertEqual(1, html.count(u'a { color: red; }'))
        self.assertEqual(0, html.count(u'<a style="color: red;">'))
        
    def test_several_args_email_mode_is_inline(self):
        template = self.template_content.format(
            'a="color: red; background: blue;" td="border: none;" img="width: 100px;"'
        )
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(0, html.count(u'<style>'))
        self.assertEqual(3, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(1, html.count(u'<img style="width: 100px;"/>'))
        self.assertEqual(4, html.count(u'<td style="border: none;">'))
        
    def test_several_args_edit_mode_is_in_style(self):
        template = self.template_content.format(
            'a="color: red; background: blue;" td="border: none;" img="width: 100px;"'
        )
        tpl = Template(template)
        html = tpl.render(Context({'by_email': False}))
        self.assertEqual(1, html.count(u'<style>'))
        self.assertEqual(1, html.count(u'a { color: red; background: blue; }'))
        self.assertEqual(1, html.count(u'img { width: 100px; }'))
        self.assertEqual(1, html.count(u'td { border: none; }'))
        self.assertEqual(0, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(0, html.count(u'<img style="width: 100px;">'))
        self.assertEqual(0, html.count(u'<td style="border: none;">'))
        
    def test_class_selector_email_mode_is_inline(self):
        template = self.template_content.format('"table.this-one td"="border: none;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(0, html.count(u'<style>'))
        self.assertEqual(0, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(0, html.count(u'<img style="width: 100px;"/>'))
        self.assertEqual(2, html.count(u'<td style="border: none;">'))

    def test_dont_overwrite_inline(self):

        template_content = """
            {{% load coop_utils %}}
            {{% nlf_css {0} %}}
                <a style="color: #fff">One</a>
                <a>Two</a>
            {{% end_nlf_css %}}
        """

        template = template_content.format('a="color: red; background: blue;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(1, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(1, html.count(u'<a style="color: #fff; background: blue;">'))

    def test_keep_order_inline(self):

        template_content = """
            {{% load coop_utils %}}
            {{% nlf_css {0} %}}
                <a style="background: #000; color: #fff;">One</a>
                <a>Two</a>
            {{% end_nlf_css %}}
        """

        template = template_content.format('a="color: red; background: blue; padding: 0;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))

        self.assertEqual(1, html.count(u'<a style="background: #000; color: #fff; padding: 0;">'))
        self.assertEqual(1, html.count(u'<a style="color: red; background: blue; padding: 0;">'))

    def test_accept_quote(self):

        template_content = """
            {{% load coop_utils %}}
            {{% nlf_css {0} %}}
                <a style="font-family: 'Arial';">One</a>
                <a>Two</a>
            {{% end_nlf_css %}}
        """

        template = template_content.format('''a="font-family: 'Tahoma';"''')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(1, html.count(u'''<a style="font-family: 'Arial';">'''))
        self.assertEqual(1, html.count(u'''<a style="font-family: 'Tahoma';">'''))

    def test_css_order(self):

        template_content = """
            {{% load coop_utils %}}
            {{% nlf_css {0} %}}
                <a>One</a>
                <div class="blue">
                <a>Two</a>
                </div>
            {{% end_nlf_css %}}
        """

        tpl = Template(template_content.format('a="color: #000" ".blue a"="color: #fff"'))
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(2, html.count(u'''<a style="color: #000;">'''))
        self.assertEqual(0, html.count(u'''<a style="color: #fff';">'''))

        tpl = Template(template_content.format('".blue a"="color: #fff" a="color: #000"'))
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(1, html.count(u'''<a style="color: #000;">'''))
        self.assertEqual(1, html.count(u'''<a style="color: #fff;">'''))

    def test_no_end_tag(self):

        template_content = """
            {{% load coop_utils %}}
            {{% nlf_css {0} %}}
                <a>One</a>
                <hr />
                <a>Two</a>
            {{% end_nlf_css %}}
        """

        template = template_content.format('hr="color: red;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(1, html.count(u'<hr style="color: red;"/>'))


class HtmlFixTest(BaseTestCase):
    """Test dirty fixs for newsletter html"""

    def test_strip_a_tags(self):
        """Make sure that the a tags have no space inside"""
        html = u'''
        <p>
         <h1>Test</h1>
         <a class="link" href="/">
          Cool
         </a>
         <div>
         <a href="/test">
          Test
         </a>
         </div>
        </p>
        '''

        soup = BeautifulSoup(html)
        fixed_html = strip_a_tags(soup.prettify())

        expected_html = '<p>\n <h1>\n  Test\n </h1>\n <a class="link" href="/">Cool</a>\n <div>\n  ' \
        '<a href="/test">Test</a>\n </div>\n</p>\n'

        self.assertEqual(fixed_html, expected_html)

    def test_no_a_tags(self):
        """Make sure that content is returned properly if no a tag in content"""
        html = u'''
        <p>
         <h1>Test</h1>
        </p>
        '''

        soup = BeautifulSoup(html)
        fixed_html = strip_a_tags(soup.prettify())

        expected_html = '<p>\n <h1>\n  Test\n </h1>\n</p>\n'

        self.assertEqual(fixed_html, expected_html)

    def test_avoid_line_too_long(self):
        """newsletter should not contains more than 998 chars. Force some endline if so"""
        html = u'''
        <p>
         <h1>Test</h1>
         <a class="link" href="/">
          {0}
         </a>
         <div>
         <a href="/test">
          <h2>Eté</h2>
         </a>
         </div>
        </p>
        '''.format('abcd ' * 300)  # 1500

        fixed_html = avoid_line_too_long(html)

        max_length = 0
        for line in fixed_html.split('\n'):
            max_length = max(max_length, len(line))

        self.assertEqual(fixed_html.count('abcd'), 300)
        self.assertEqual(len(fixed_html.splitlines()), len(html.splitlines()) + 1 + (1500 // 900))
        self.assertTrue(max_length <= 900)
        self.assertTrue(fixed_html.find(u'<h1>Test</h1>') > 0)
        self.assertTrue(fixed_html.find(u'<h2>Eté</h2>') > 0)

    def test_avoid_line_very_too_long(self):
        """newsletter should not contains more than 998 chars. Force some endline if so"""
        html = u'''
        <p>
         <h1>Test</h1>
         <a class="link" href="/">
          {0}
         </a>
         <div>
         <a href="/test">
          <h2>Eté</h2>
         </a>
         </div>
        </p>
        '''.format(u'abcdé ' * 600)  # 4200 characters

        fixed_html = avoid_line_too_long(html)

        max_length = 0
        for line in fixed_html.split('\n'):
            max_length = max(max_length, len(line))

        self.assertEqual(fixed_html.count(u'abcdé'), 600)
        self.assertEqual(len(fixed_html.splitlines()), len(html.splitlines()) + 1 + (4200 // 900))
        self.assertTrue(max_length <= 900)
        self.assertTrue(fixed_html.find(u'<h1>Test</h1>') > 0)
        self.assertTrue(fixed_html.find(u'<h2>Eté</h2>') > 0)

    def test_avoid_line_too_long_no_change(self):
        """newsletter should not contains more than 998 chars. Force some endline if so"""
        html = u'''
        <p>
         <h1>Test</h1>
         <a class="link" href="/">
          Cool
         </a>
         <div>
         <a href="/test">
          Test
         </a>
         </div>
        </p>
        '''

        fixed_html = avoid_line_too_long(html)
        self.assertEqual(fixed_html, html)



# -*- coding: utf-8 -*-
"""forms"""

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

import floppyforms

from coop_cms.forms.fields import HidableMultipleChoiceField
from coop_cms.forms.navigation import WithNavigationModelForm
from coop_cms.models import Link, Document, MediaFilter, ImageSize

from coop_cms.widgets import ChosenSelectMultiple


class MediaBaseAddMixin(object):

    def __init__(self, *args, **kwargs):
        super(MediaBaseAddMixin, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        # Media filters
        queryset1 = MediaFilter.objects.all()
        if queryset1.count():
            self.fields['filters'].choices = [(x.id, x.name) for x in queryset1]
            try:
                self.fields['filters'].widget = ChosenSelectMultiple(
                    choices=self.fields['filters'].choices, force_template=True,
                )
            except NameError:
                # print 'No ChosenSelectMultiple'
                pass
        else:
            self.fields['filters'].widget = self.fields['filters'].hidden_widget()

    def clean_filters(self):
        """validation"""
        filters = self.cleaned_data['filters']
        return [MediaFilter.objects.get(id=pk) for pk in filters]


class AddImageForm(MediaBaseAddMixin, floppyforms.Form):
    """Form for adding new image"""

    image = floppyforms.ImageField(required=True, label=_('Image'),)
    descr = floppyforms.CharField(
        required=False,
        widget=floppyforms.TextInput(
            attrs={'size': '35', 'placeholder': _(u'Optional description'), }
        ),
        label=_('Description'),
    )
    copyright = floppyforms.CharField(required=False, label=_(u'copyright'))
    filters = HidableMultipleChoiceField(
        required=False, label=_(u"Filters"), help_text=_(u"Choose between tags to find images more easily")
    )
    size = floppyforms.ChoiceField(
        required=False, label=_(u"Size"), help_text=_(u"Define a size if you want to resize the image")
    )

    def __init__(self, *args, **kwargs):
        super(AddImageForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        img_size_queryset = ImageSize.objects.all()
        if img_size_queryset.count():
            self.fields['size'].choices = [
                (u'', u'')
            ] + [
                (img_size.id, unicode(img_size)) for img_size in img_size_queryset
            ]
        else:
            self.fields['size'].widget = floppyforms.HiddenInput()

    def clean_size(self):
        """validation"""
        size_id = self.cleaned_data['size']
        if not size_id:
            return None
        try:
            return ImageSize.objects.get(id=size_id)
        except (ValueError, ImageSize.DoesNotExist):
            raise ValidationError(_(u"Invalid choice"))


class AddDocForm(MediaBaseAddMixin, forms.ModelForm):
    """add document form"""

    filters = floppyforms.MultipleChoiceField(
        required=False, label=_(u"Filters"), help_text=_(u"Choose between tags to find images more easily")
    )

    class Meta:
        model = Document
        fields = ('file', 'name', 'is_private', 'category')


class NewLinkForm(WithNavigationModelForm):
    """New link form"""
    class Meta:
        model = Link
        fields = ('title', 'url',)



# -*- coding: utf-8 -*-
"""
Applicaion for unit testing
"""

from django import VERSION

if VERSION > (1, 7, 0):
    from django.apps import AppConfig

    class CoopCmsTestAppConfig(AppConfig):
        name = 'coop_cms.apps.test_app'
        verbose_name = "coop CMS > App for unit test"



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'ArticleSection.ordering'
        db.add_column('coop_cms_articlesection', 'ordering', self.gf('django.db.models.fields.IntegerField')(default=0), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'ArticleSection.ordering'
        db.delete_column('coop_cms_articlesection', 'ordering')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'coop_cms.articlesection': {
            'Meta': {'object_name': 'ArticleSection'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'coop_cms.document': {
            'Meta': {'object_name': 'Document'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_private': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.image': {
            'Meta': {'object_name': 'Image'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.link': {
            'Meta': {'object_name': 'Link'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'coop_cms.navnode': {
            'Meta': {'unique_together': "(('content_type', 'object_id'),)", 'object_name': 'NavNode'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_navigation': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'ordering': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'default': '0', 'to': "orm['coop_cms.NavNode']", 'null': 'True', 'blank': 'True'})
        },
        'coop_cms.navtree': {
            'Meta': {'object_name': 'NavTree'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'coop_cms.navtype': {
            'Meta': {'object_name': 'NavType'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'unique': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label_rule': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'search_field': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.newsletter': {
            'Meta': {'object_name': 'Newsletter'},
            'content': ('html_field.db.models.fields.HTMLField', [], {'default': "'<br>'", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['coop_cms.NewsletterItem']", 'symmetrical': 'False', 'blank': 'True'}),
            'subject': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'template': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.newsletteritem': {
            'Meta': {'unique_together': "(('content_type', 'object_id'),)", 'object_name': 'NewsletterItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'coop_cms.newslettersending': {
            'Meta': {'object_name': 'NewsletterSending'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'newsletter': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['coop_cms.Newsletter']"}),
            'scheduling_dt': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'sending_dt': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'})
        },
        'coop_cms.pieceofhtml': {
            'Meta': {'object_name': 'PieceOfHtml'},
            'content': ('html_field.db.models.fields.HTMLField', [], {'default': "''", 'blank': 'True'}),
            'div_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['coop_cms']
